package com.niko.pg.engine;

import java.util.Comparator;

import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Sort;
import com.niko.pg.engine.entity.Entity;

public class Layer {

	private World world;
	
	private Array<Entity> entities;
	private Array<Entity> addable;
	private Array<Entity> removable;
	
	//float parallax = 1f;

	float parallaxX = 1f;
	float parallaxY = 1f;
	
	public Layer(World world) {
		this.world = world;
		entities = new Array<>();
		addable = new Array<>();
		removable = new Array<>();
	}
	
	public Layer(World world, float parallaxX, float parallaxY) {
		this(world);
		this.parallaxX = parallaxX;
		this.parallaxY = parallaxY;
	}
	
	/* ------------------------ */
	/* update, render and debug */
	/* ------------------------ */
	
	/**
	 * update this layer
	 * @param delta
	 */
	public void update(float delta) {
		// add new entities
		if (addable.size > 0) {
			for (Entity entity : addable) {
				entities.add(entity);
				entity.addedToWorld();
			}
			addable.clear();
			entities.sort(Entity.comparator);
		}
	
		// update all the entities
		for (Entity entity : entities) {
			if (!entity.active)
				continue;
			entity.update(delta);
			entity.checkWorldBoundaries();
		}
	
		// remove signed entities
		for (Entity entity : removable) {
			entities.removeValue(entity, true);
			entity.removedFromWorld();
		}
		removable.clear();
	}
	
	/**
	 * render this layer
	 * @return the amount of entities that got rendered
	 */
	public int render(SpriteBatch batch) {
		int renderedEntities = 0;
		for (Entity e : entities) {
			if (!e.visible)
				continue;
			e.render(batch);
			renderedEntities++;
		}
		return renderedEntities;
	}
	
	/**
	 * debug this layer
	 * @param shapeRenderer
	 */
	public void debug(ShapeRenderer shapeRenderer) {
		for (Entity entity : entities) {
			entity.debug(shapeRenderer);
		}
	}
	
	/* --------------- */
	/* utility methods */
	/* --------------- */
	
	/**
	 * Add entity to this layer and sort the entities in this layer based on their z order
	 * @param e
	 *            entity to add
	 */
	public void add(Entity e) {
		e.setWorld(world);
		addable.add(e);
	}
	
	public void addAll(Entity... entities) {
		for (Entity e : entities) {
			add(e);
		}
	}
	
	/**
	 * @param entity
	 *            to remove from game
	 * @return false if entity is already set to be remove
	 */
	public boolean remove(Entity entity) {
		if (removable.contains(entity, true))
			return false;
		removable.add(entity);
		return true;
	}
	
	public Array<Entity> getEntities() {
		return new Array<>(entities);
	}
	
	public static Comparator<Layer> comparator = new Comparator<Layer>() {
		@Override
		public int compare(Layer o1, Layer o2) {
			// return Float.compare(o1.parallax, o2.parallax);
			
			int result = Float.compare(o1.parallaxX, o2.parallaxX);
			return (result != 0) ? result : Float.compare(o1.parallaxY, o2.parallaxY);
		}
	};
	
}
