package com.niko.pg.engine;

import com.badlogic.gdx.ApplicationListener;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;

/** <p>
 * An {@link ApplicationListener} that delegates to a {@link Screen}. This allows an application to easily have multiple screens.
 * </p>
 * <p>
 * Screens are not disposed automatically. You must handle whether you want to keep screens around or dispose of them when another
 * screen is set.
 * </p> */
public abstract class StateBasedGame implements ApplicationListener {
	private State state;

	@Override
	public void dispose() {
		if (state != null) state.hide();
	}

	@Override
	public void pause() {
		if (state != null) state.pause();
	}

	@Override
	public void resume() {
		if (state != null) state.resume();
	}

	@Override
	public void render() {
		if (state != null) state.render(Gdx.graphics.getDeltaTime());
	}

	@Override
	public void resize(int width, int height) {
		if (state != null) state.resize(width, height);
	}

	/** Sets the current screen. {@link Screen#hide()} is called on any old screen, and {@link Screen#show()} is called on the new
	 * screen. */
	public void setState(State state) {
		if (this.state != null) this.state.hide();
		this.state = state;
		state.show();
		state.resize(Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
	}

	/** @return the currently active {@link Screen}. */
	public State getState() {
		return state;
	}
}
