package com.niko.pg.engine;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.utils.Array;

import com.niko.pg.Game;
import com.niko.pg.engine.entity.Entity;
import com.niko.pg.screens.GameState;

/**
 * 
 * @author Niko
 * 
 */
public abstract class World extends GameState {

	public enum Levels {
		BG, ACTIVE, FG, HUD
	};

	private Array<Layer> background = new Array<>();
	private Array<Layer> foreground = new Array<>();
	private Layer activeLayer = new Layer(this);
	private Layer hud = new Layer(this);

	public int width;
	public int height;
	public int renderedEntities;

	public World(Game game, int width, int height) {
		super(game);
		this.width = width;
		this.height = height;
	}

	@Override
	public void show() {
		init();
	}

	public abstract void init();

	@Override
	public void render(float delta) {
		// call the update method to update the world
		update(delta);
		// render all the entities
		game.worldRenderer.renderWorld(this);
	}

	@Override
	public void resize(int width, int height) {
		game.worldRenderer.resize(width, height);
	}

	/* ------------------------ */
	/* update, render and debug */
	/* ------------------------ */

	public void update(float delta) {

		for (Layer layer : background)
			layer.update(delta);
		
		activeLayer.update(delta);
		
		for (Layer layer : foreground)
			layer.update(delta);
		
		hud.update(delta);
		
	}

	public void render(SpriteBatch batch, ParallaxCamera cam) {
		renderedEntities = 0;
		
		for (Layer layer : background) {
			batch.setProjectionMatrix(cam.calculateParallaxMatrix(layer.parallaxX, layer.parallaxY));
			batch.begin();
			renderedEntities += layer.render(batch);
			batch.end();
		}

		batch.setProjectionMatrix(cam.calculateParallaxMatrix(activeLayer.parallaxX, activeLayer.parallaxY));
		batch.begin();
		renderedEntities += activeLayer.render(batch);
		batch.end();
		
		for (Layer layer : foreground) {
			batch.setProjectionMatrix(cam.calculateParallaxMatrix(layer.parallaxX, layer.parallaxY));
			batch.begin();
			renderedEntities += layer.render(batch);
			batch.end();
		}

		batch.setProjectionMatrix(cam.calculateParallaxMatrix(0, 0));
		batch.begin();
		renderedEntities += hud.render(batch);
		batch.end();
		
	}

	public void debug(ShapeRenderer renderer, ParallaxCamera cam) {
		
		for (Layer layer : background) {
			renderer.setProjectionMatrix(cam.calculateParallaxMatrix(layer.parallaxX, layer.parallaxY));
			layer.debug(renderer);
		}

		renderer.setProjectionMatrix(cam.calculateParallaxMatrix(activeLayer.parallaxX, activeLayer.parallaxY));
		activeLayer.debug(renderer);
		
		for (Layer layer : foreground) {
			renderer.setProjectionMatrix(cam.calculateParallaxMatrix(layer.parallaxX, layer.parallaxY));
			layer.debug(renderer);
		}

		renderer.setProjectionMatrix(cam.calculateParallaxMatrix(0, 0));
		hud.debug(renderer);
	}

	/* --------------- */
	/* utility methods */
	/* --------------- */

	public void add(Entity e) {
		activeLayer.add(e);
	}

	private void addToBg(Entity e, float parallaxX, float parallaxY) {

		for (Layer layer : background) {
			if (layer.parallaxX > parallaxX)
				break;
			else if (layer.parallaxX == parallaxX) {
				layer.add(e);
				return;
			}
		}

		Layer newLayer = new Layer(this, parallaxX, parallaxY);
		newLayer.add(e);
		background.add(newLayer);
		background.sort(Layer.comparator);
	}

	private void addToFg(Entity e, float parallaxX, float parallaxY) {

		for (Layer layer : foreground) {
			if (layer.parallaxX > parallaxX)
				break;
			else if (layer.parallaxX == parallaxX) {
				layer.add(e);
				return;
			}
		}

		Layer newLayer = new Layer(this, parallaxX, parallaxY);
		newLayer.add(e);
		foreground.add(newLayer);
		foreground.sort(Layer.comparator);
	}

	private void addToHud(Entity e) {
		// TODO
	}

	public void add(Entity e, float parallaxX, float parallaxY) {
		if (parallaxX == 1f) {
			add(e);
			Gdx.app.log("DEBUG", "The entity " + e.getName() + " was added to the active layer");
		}
		else if (parallaxX > 1f) {
			add(e, Levels.FG, parallaxX, parallaxY);
			Gdx.app.log("DEBUG", "The entity " + e.getName() + " was added to a foreground layer with a parallax value of " + parallaxX);
		}
		else {
			add(e, Levels.BG, Math.max(parallaxX, 0), Math.max(parallaxY, 0));
			Gdx.app.log("DEBUG", "The entity " + e.getName() + " was added to a background layer with a parallax value of " + parallaxX);
		}
	}

	/**
	 * Add an entity to a specific layer depth in this world </p> note that for
	 * the active layer and the hud, the parallax argument doesn't do anything,<br>
	 * it's only used when choosing to add to either the background or the
	 * foreground layers.
	 * 
	 * @param e
	 * @param lvl
	 * @param parallax
	 */
	public void add(Entity e, Levels lvl, float parallaxX, float parallaxY) {
		switch (lvl) {
		case BG:
			addToBg(e, parallaxX, parallaxY);
			break;
		case ACTIVE:
			add(e);
			break;
		case FG:
			addToFg(e, parallaxX, parallaxY);
			break;
		case HUD:
			addToHud(e);
			break;
		}
	}

	public boolean remove(Entity entity) {
		// TODO
		return false;
	}

	public Array<Entity> getEntities() {
		return activeLayer.getEntities();
	}
	
	public void setEntityToFollow(Entity entity) {
		game.worldRenderer.cam.setFollow(entity);
	}

}
