package com.niko.pg.engine.baseEntities;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;

import com.niko.pg.engine.entity.Entity;

public abstract class MovableEntity extends Entity {

	public Vector2 speed = new Vector2(0, 0);
	public Vector2 maxSpeed = new Vector2(4, 8);
	public Vector2 acceleration = new Vector2(0, 0);
	public Vector2 friction = new Vector2(0.2f, 0.2f);
	public float gravity = 9.8f;
	public int stepSize = 1;

	public MovableEntity(String name, int x, int y, int width, int height) {
		super(name, x, y);
		setSize(width, height);
	}
	
	public MovableEntity(String name, int x, int y, int width, int height, Texture texture) {
		super(name, x, y, texture);
		setSize(width, height);
	}

	public void move(boolean moveX, boolean moveY) {		
		// if we should move horizontally
		if (moveX) {
			// make us move, and stop us on collision
			if (!motionx(speed.x)) {
				speed.x = 0;
			}

			// increase velocity/speed
			speed.x += acceleration.x;
		}

		// if we should move vertically
		if (moveY) {
			// make us move, and stop us on collision
			if (!motiony(speed.y)) {
				speed.y = 0;
			}

			// increase velocity/speed
			speed.y += acceleration.y;
		}
	}

	public void maxspeed(boolean mx, boolean my) {
		if (mx)
			if (Math.abs(speed.x) > maxSpeed.x)
				speed.x = maxSpeed.x * Math.signum(speed.x);
		if (my)
			if (Math.abs(speed.y) > maxSpeed.y)
				speed.y = maxSpeed.y * Math.signum(speed.y);
	}

	public void gravity(float delta) {
		// increase velocity/speed based on gravity
		speed.y -= gravity * delta;
	}

	public void friction(boolean mx, boolean my) {
		// if we should use friction, horizontally
		if (mx) {
			// speed > 0, then slow down
			if (speed.x > 0) {
				speed.x -= friction.x;
				// if we go below 0, stop.
				if (speed.x < 0) {
					speed.x = 0;
				}
			}
			// speed < 0, then "speed up" (in a sense)
			if (speed.x < 0) {
				speed.x += friction.x;
				// if we go above 0, stop.
				if (speed.x > 0) {
					speed.x = 0;
				}
			}
		}
		// if we should use friction, vertically
		if (my) {
			// speed > 0, then slow down
			if (speed.y > 0) {
				speed.y -= friction.y;
				// if we go below 0, stop.
				if (speed.y < 0) {
					speed.y = 0;
				}
			}
			// speed < 0, then "speed up" (in a sense)
			if (speed.y < 0) {
				speed.y += friction.y;
				// if we go above 0, stop.
				if (speed.y > 0) {
					speed.y = 0;
				}
			}
		}
	}

	public boolean motionx(float spdx) {
		// for each pixel that we will move...
		for (int i = 0; i < Math.abs(spdx); i++) {

			// for checking if whether we moved or not
			boolean moved = false;
			boolean below = false;
			
			// check we are on the ground
			if (collide(SOLID, getX(), getY() - 1) != null) {
				below = true;
			}
			
			// run through how high a slope we can move up
			for (int s = 0; s <= stepSize; s++) {
				// if we don't hit a solid in the direction we're moving
				if (collide(SOLID, getX() + Math.signum(spdx), getY() + s) == null) {
					// increase/decrease positions
					setX(getX() + Math.signum(spdx));
					// move up the slope, set that we moved and stop stepping up
					setY(getY() + s);
					moved = true;
					break;
				}
			}
			
			// if we are now in the air, but just above a platform, move us
			// down.
			if (below && collide(SOLID, getX(), getY() - 1) == null)
				setY(getY() - 1);
			
			// if we haven't moved, return false to stop
			if (!moved) {
				return false;
			}
			
		}
		// hit nothing
		return true;
	}

	/**
	 * translate the entity on the y axis
	 * @param spdy amount of pixels to translate (speed y) 
	 * @return returns false if a collision occurred on the way
	 */
	public boolean motiony(float spdy) {
		// for each pixel that we will move...
		for (int i = 0; i < Math.abs(spdy); i++) {
			// if we collide with some entity then stop the movement
			if (collide(SOLID, getX(), getY() + Math.signum(spdy)) != null)
				return false; // hit solid
			setY(getY() + Math.signum(spdy));
		}
		// hit nothing
		return true;
	}

}
