package com.niko.pg.engine.baseEntities;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

import com.niko.pg.engine.entity.Entity;

public class Slope extends Solid {

	boolean oneWay;

	public Slope(String name) {
		this(name, 0, 0, 0, 0, true);
	}

	public Slope(String name, boolean oneWay) {
		this(name, 0, 0, 0, 0, oneWay);
	}

	public Slope(String name, float x1, float y1, float x2, float y2) {
		this(name, x1, y1, x2, y2, true);
	}

	public Slope(String name, float x1, float y1, float x2, float y2,
			boolean oneWay) {
		super(name);
		this.setPoints(x1, y1, x2, y2);
		this.oneWay = oneWay;
	}

	public void setPoints(Vector2 p1, Vector2 p2) {
		this.setPoints(p1.x, p1.y, p2.x, p2.y);
	}

	public void setPoints(float x1, float y1, float x2, float y2) {
		if (x1 < x2) {
			float width = x2 - x1;
			float height = y2 - y1;
			setBounds(x1, y1, width, height);
		} else {
			float width = x1 - x2;
			float height = y1 - y2;
			setBounds(x2, y2, width, height);
		}
	}
	
	@Override
	public Rectangle getBoundingRectangle() {
		Rectangle bounds = super.getBoundingRectangle();
		
		// if the slope has an angle of zero, then scale the bounds a little
		// in order for the collision to work
		if (bounds.height == 0) {
			bounds.y -= 5;
			bounds.height = 10;
		}

		return bounds;
		
	}
	
	@Override
	public boolean isColliding(Entity e, float x, float y) {
		
		// if this slope is a one way slope
		// and the entity colliding is a movable entity
		if (oneWay && e instanceof MovableEntity) {
			MovableEntity moveable = (MovableEntity) e;
			// check if the entity is moving upwards
			if (moveable.speed.y > 0)
				return false; // if it does then go trough the slope
		}

		x += e.getOriginX();
		y += e.getOriginY();

		Rectangle bounds = getBoundingRectangle();
		
		
		
		if (bounds.x > x || bounds.x + bounds.width < x || bounds.y > y
				|| bounds.y + bounds.height < y)
			return false;
		
		Vector2 p1 = new Vector2(getX(), getY());
		Vector2 p2 = new Vector2(getX() + getWidth(), getY() + getHeight());

		if (isLeft(p1, p2, new Vector2(x, y + 2))
				&& isRight(p1, p2, new Vector2(x, y)))
			return true;

		return false;
	}

	/**
	 * checks if a point is from the left of a line
	 * 
	 * @param p1
	 *            the first point that represents the line
	 * @param p2
	 *            the second point that represents the line
	 * @param p3
	 *            the point that is tested
	 * @return true if p3 is from the left of the line created by the two points
	 *         p1 and p2
	 */
	public boolean isLeft(Vector2 p1, Vector2 p2, Vector2 p3) {
		return ((p2.x - p1.x) * (p3.y - p1.y) - (p2.y - p1.y) * (p3.x - p1.x)) > 0;
	}

	/**
	 * checks if a point is from the right of a line
	 * 
	 * @param p1
	 *            the first point that represents the line
	 * @param p2
	 *            the second point that represents the line
	 * @param p3
	 *            the point that is tested
	 * @return true if p3 is from the right of the line created by the two
	 *         points p1 and p2
	 */
	public boolean isRight(Vector2 p1, Vector2 p2, Vector2 p3) {
		return ((p2.x - p1.x) * (p3.y - p1.y) - (p2.y - p1.y) * (p3.x - p1.x)) < 0;
	}

	@Override
	public void debug(ShapeRenderer shapeRenderer) {
		Color red = new Color(1f, 0, 0, 0.8f);
		drawBounds(shapeRenderer, new Color(1, 0.4f, 0, 0.2f));
		drawLine(shapeRenderer, getX(), getY(), getX() + getWidth(), getY()
				+ getHeight(), red);
		drawPoint(shapeRenderer, getX(), getY(), red, 2.5f);
		drawPoint(shapeRenderer, getX() + getWidth(), getY() + getHeight(),
				red, 2.5f);

	}

	/**
	 * @return the angle created between the slope and the x axis
	 */
	public float slopeAngle() {
		return (float) Math.toDegrees(Math.atan(getHeight() / getWidth()));
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Slope [name=");
		builder.append(name);
		builder.append(", x=");
		builder.append(getX());
		builder.append(", y=");
		builder.append(getY());
		builder.append(", width=");
		builder.append(getWidth());
		builder.append(", height=");
		builder.append(getHeight());
		builder.append("]");
		return builder.toString();

	}

}
