package com.niko.pg.engine.entity;

import java.util.Comparator;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.collision.BoundingBox;
import com.badlogic.gdx.utils.Array;

import com.niko.pg.engine.World;

/**
 * An Entity abstracts the Sprite class and gives more control over it
 * 
 * @author Niko
 * 
 */
public abstract class Entity extends Sprite {

	public static final String SOLID = "Solid";
	public static final String PLAYER = "Player";
	
	protected World world;

	protected String name;

	public boolean active = true;
	public boolean collidable = true;
	public boolean visible = true;
	
	protected Array<String> collisionTypes = new Array<String>();
	
	public int depth = -1;

	public Entity(String name) {
		this(name, 0, 0);
	}

	public Entity(String name, float x, float y) {
		super();
		setPosition(x, y);
		this.name = name;
	}

	public Entity(String name, Sprite sprite) {
		super(sprite);
		this.name = name;
	}

	public Entity(String name, Texture texture) {
		this(name, 0, 0, texture);
	}
	
	public Entity(String name, float x, float y, Texture texture) {
		super(texture);
		setPosition(x, y);
		this.name = name;
	}

	public Entity(String name, TextureRegion region) {
		this(name, 0, 0, region);
	}

	public Entity(String name, float x, float y, TextureRegion region) {
		super(region);
		setPosition(x, y);
		this.name = name;
	}
	
	
	public void update(float delta) {

	}

	public void render(SpriteBatch batch) {
		// draw the entity
		if (getTexture() != null)
			draw(batch);
	}

	public abstract void debug(ShapeRenderer shapeRenderer);

	protected void drawBounds(ShapeRenderer shapeRenderer) {
		drawBounds(shapeRenderer, new Color(1, 0, 0, 0.8f));
	}

	protected void drawBounds(ShapeRenderer shapeRenderer, Color color) {
		shapeRenderer.begin(ShapeType.Rectangle);
		shapeRenderer.setColor(color);
		shapeRenderer.identity();
		Rectangle bounds = getBoundingRectangle();
		shapeRenderer.rect(bounds.x, bounds.y, bounds.width, bounds.height);
		shapeRenderer.end();
	}

	protected void drawCenter(ShapeRenderer shapeRenderer) {
		drawPoint(shapeRenderer, getX() + getWidth() / 2f, getY() + getHeight()
				/ 2f, new Color(0, 0, 0, 0.8f), 1f);
	}

	protected void drawOrigin(ShapeRenderer shapeRenderer) {
		drawOrigin(shapeRenderer, new Color(1f, 0, 1f, 0.6f), 2.5f);
	}

	protected void drawOrigin(ShapeRenderer shapeRenderer, Color color) {
		drawPoint(shapeRenderer, getX() + getOriginX(), getY() + getOriginY(),
				color, 2.5f);
	}

	protected void drawOrigin(ShapeRenderer shapeRenderer, Color color, float size) {
		drawPoint(shapeRenderer, getX() + getOriginX(), getY() + getOriginY(),
				color, size);
	}

	protected void drawPoint(ShapeRenderer shapeRenderer, float x, float y,
			Color color, float radius) {
		shapeRenderer.begin(ShapeType.FilledCircle);
		shapeRenderer.setColor(color);
		shapeRenderer.identity();
		shapeRenderer.translate(x, y, 0);
		shapeRenderer.filledCircle(0, 0, radius);
		shapeRenderer.end();
	}
	
	protected void drawLine(ShapeRenderer shapeRenderer, float x1, float y1, float x2, float y2,
			Color color) {
		shapeRenderer.begin(ShapeType.Line);
		shapeRenderer.setColor(color);
		shapeRenderer.identity();
		shapeRenderer.line(x1, y1, x2, y2);
		shapeRenderer.end();
	}

	public void setWorld(World world) {
		this.world = world;
	}

	/**
	 * overload if you want to act on addition to the world
	 */
	public void addedToWorld() {
	}

	/**
	 * overload if you want to act on removal from the world
	 */
	public void removedFromWorld() {
	}

	/**
	 * Response to a collision with another entity
	 * 
	 * @param other
	 *            The other entity that collided with us.
	 */
	public void collisionResponse(Entity other) {
	}

	/**
	 * overload if you want to act on leaving world boundaries
	 */
	public void leftWorldBoundaries() {
	}
	
	/**
	 * Check for a collision with another entity.
	 * </p>
	 * Two entities collide when the bounds of this entity overlaps with
	 * the bounds of another entity.
	 * </p>
	 * The bounds for the other entity starts at the provided x,y coordinates.
	 * </p>
	 * overload to add specific collision detection for this entity
	 */
	public boolean isColliding(Entity e, float x, float y) {
		Rectangle bounds = e.getBoundingRectangle();
		bounds.x += x - e.getX();
		bounds.y += y - e.getY();
		if (bounds.overlaps(getBoundingRectangle()))
			return true;
		return false;
	}
	
	public static Comparator<Entity> comparator = new Comparator<Entity>() {
		@Override
		public int compare(Entity e1, Entity e2) {
			return Integer.compare(e1.depth, e2.depth);
		}
	};
	
	
	/**
	 * Add a type that this entity can collide with. To allow collision with
	 * other entities add at least 1 type. For example in a space invaders game.
	 * To allow a ship to collide with a bullet and a monster:
	 * ship.addType("bullet", "monster")
	 * 
	 * @param types
	 *            The types that this entity can collide with.
	 */
	public void addType(String... types) {
		collisionTypes.addAll(types);
	}
	
	/**
	 * check if this entity is of the given collision type
	 * @param type
	 * @return  true if the collision types array contains the given type
	 */
	public boolean isType(String type) {
		return collisionTypes.contains(type, false);
	}
	
	/**
	 * checks if the entity is one of the following collision types
	 * @param types
	 * @return true true if the collision types array contains at least one of the given types
	 */
	public boolean isOneOfTypes(String[] types) {
		for (String type : types) {
			if (isType(type))
				return true;
		}
		return false;
	}

	/**
	 * Reset the types that this entity can collide with
	 */
	public void clearTypes() {
		collisionTypes.clear();
	}
	
	/**
	 * Check for a collision with another entity of the given entity type.
	 * <p/>
	 * If a collision occurred then both the entities are notified of the
	 * collision by the {@link #collisionResponse(Entity)} method.
	 * 
	 * @param type
	 *            The type of another entity to check for collision.
	 * @param x
	 *            The x coordinate where the the collision check needs to be
	 *            done.
	 * @param y
	 *            The y coordinate where the the collision check needs to be
	 *            done.
	 * @return The first entity that is colliding with this entity at the x,y
	 *         coordinates, or NULL if there is no collision.
	 */
	public Entity collide(String type, float x, float y) {
		if (type == null || type.isEmpty())
			return null;
		
		for (Entity entity : world.getEntities()) {
			if (entity.collidable && entity.isType(type) && !entity.equals(this)) {
				if (entity.isColliding(this, x, y)) {
					this.collisionResponse(entity);
					entity.collisionResponse(this);
					return entity;
				}
			}
		}
		return null;
	}

	public void checkWorldBoundaries() {
		if ((getX() + getWidth()) < 0)
			leftWorldBoundaries();
		if (getX() > this.world.width)
			leftWorldBoundaries();
		if ((getY() + getHeight()) < 0)
			leftWorldBoundaries();
		if (getY() > this.world.height)
			leftWorldBoundaries();
	}

	public void set(Entity entity) {
		super.set(entity);
	}
	
	public String getName() {
		return name;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Entity [name=");
		builder.append(name);
		builder.append(", x=");
		builder.append(getX());
		builder.append(", y=");
		builder.append(getY());
		builder.append(", width=");
		builder.append(getWidth());
		builder.append(", height=");
		builder.append(getHeight());
		builder.append(", active=");
		builder.append(active);
		builder.append(", collidable=");
		builder.append(collidable);
		builder.append(", visible=");
		builder.append(visible);
		builder.append(", depth=");
		builder.append(depth);
		builder.append("]");
		return builder.toString();
	}

}
